@@ -2,17 +2,18 @@ from typing import Any, Dict
 
 
 class Messaging:
+
     def __init__(self, broker_properties: dict):
         self.broker_properties = broker_properties
 
     def connect(self):
-        raise NotImplementedError
+        raise NotImplementedError from None
 
     def disconnect(self):
-        raise NotImplementedError
+        raise NotImplementedError from None
 
     def receive_message(self, timeout_ms, queue_id: str):
-        raise NotImplementedError
+        raise NotImplementedError from None
 
     def send_message(
         self,
@@ -21,4 +22,4 @@ class Messaging:
         user_properties: Dict = None,
         user_context: Dict = None,
     ):
-        raise NotImplementedError
+        raise NotImplementedError from None
