@@ -10,12 +10,14 @@ from enum import Enum
 from solace.messaging.messaging_service import (
     MessagingService,
     ReconnectionListener,
+    ServiceEvent,
     ReconnectionAttemptListener,
     ServiceInterruptionListener,
-    ServiceEvent,
 )
+from solace.messaging.resources.topic_subscription import TopicSubscription
+from solace.messaging.resources.topic import Topic
 from solace.messaging.resources.queue import Queue
-from solace.messaging.config.retry_strategy import RetryStrategy
+from solace.messaging.receiver.message_receiver import MessageHandler, InboundMessage
 from solace.messaging.receiver.persistent_message_receiver import (
     PersistentMessageReceiver,
 )
@@ -24,13 +26,11 @@ from solace.messaging.publisher.persistent_message_publisher import (
     MessagePublishReceiptListener,
     PublishReceipt,
 )
-from solace.messaging.receiver.message_receiver import MessageHandler, InboundMessage
-from solace.messaging.errors.pubsubplus_client_error import PubSubPlusClientError
+from solace.messaging.config.retry_strategy import RetryStrategy
 from solace.messaging.config.missing_resources_creation_configuration import (
     MissingResourcesCreationStrategy,
 )
-from solace.messaging.resources.topic_subscription import TopicSubscription
-from solace.messaging.resources.topic import Topic
+from solace.messaging.errors.pubsubplus_client_error import PubSubPlusClientError
 from solace import SOLACE_LOGGING_CONFIG
 
 from .messaging import Messaging
@@ -101,18 +101,22 @@ class ServiceEventHandler(
     ):
         self.stop_signal = stop_signal
         self.error_prefix = error_prefix
-        self.strategy = strategy
         self.retry_count = retry_count
         self.retry_interval = retry_interval
         self.connection_properties = connection_properties
 
+        try:
+            self.strategy = ConnectionStrategy(strategy)
+        except ValueError as e:
+            log.error(
+                f"{self.error_prefix} Invalid reconnection strategy: {strategy}. Using default strategy.",
+                trace=e,
+            )
+            self.strategy = ConnectionStrategy.FOREVER_RETRY
+
     def on_reconnected(self, service_event: ServiceEvent):
         change_connection_status(self.connection_properties, ConnectionStatus.CONNECTED)
         log.info(f"{self.error_prefix} Successfully reconnected to broker.")
-        log.debug(
-            f"{self.error_prefix} Message: %s",
-            service_event.get_message(),
-        )
 
     def on_reconnecting(self, event: "ServiceEvent"):
         change_connection_status(
@@ -127,10 +131,7 @@ class ServiceEventHandler(
                 == ConnectionStatus.RECONNECTING
             ):
                 # update retry count
-                if (
-                    self.strategy
-                    and self.strategy == ConnectionStrategy.PARAMETRIZED_RETRY
-                ):
+                if self.strategy == ConnectionStrategy.PARAMETRIZED_RETRY:
                     if self.retry_count <= 0:
                         log.error(
                             f"{self.error_prefix} Reconnection attempts exhausted. Stopping..."
@@ -139,13 +140,8 @@ class ServiceEventHandler(
                     else:
                         self.retry_count -= 1
 
-                log.error(
-                    f"{self.error_prefix} Reconnecting to broker: %s",
-                    event.get_cause(),
-                )
-                log.debug(
-                    f"{self.error_prefix} Message: %s",
-                    event.get_message(),
+                log.info(
+                    f"{self.error_prefix} Reconnecting to broker...",
                 )
                 self.stop_signal.wait(timeout=self.retry_interval / 1000)
 
@@ -156,11 +152,7 @@ class ServiceEventHandler(
         change_connection_status(
             self.connection_properties, ConnectionStatus.DISCONNECTED
         )
-        log.error(
-            f"{self.error_prefix} Service interrupted - Error cause: %s",
-            event.get_cause(),
-        )
-        log.debug(f"{self.error_prefix} Message: %s", event.get_message())
+        log.error(f"{self.error_prefix} Service interrupted")
 
 
 def set_python_solace_log_level(level: str):
@@ -223,10 +215,26 @@ class SolaceMessaging(Messaging):
                 or os.path.dirname(certifi.where())
                 or "/usr/share/ca-certificates/mozilla/",
             }
-            strategy = self.broker_properties.get("reconnection_strategy")
+
+            try:
+                if "reconnection_strategy" in self.broker_properties:
+                    strategy = ConnectionStrategy(
+                        self.broker_properties.get("reconnection_strategy")
+                    )
+                else:
+                    log.info(
+                        f"{self.error_prefix} reconnection_strategy not provided, using default value of forever_retry"
+                    )
+                    strategy = ConnectionStrategy.FOREVER_RETRY
+            except ValueError:
+                log.warning(
+                    f"{self.error_prefix} Invalid reconnection strategy: {self.broker_properties.get('reconnection_strategy')}. Using default Forever Retry strategy."
+                )
+                strategy = ConnectionStrategy.FOREVER_RETRY
+
             retry_interval = 3000  # default
             retry_count = 20  # default
-            if strategy and strategy == ConnectionStrategy.FOREVER_RETRY:
+            if strategy == ConnectionStrategy.FOREVER_RETRY:
                 retry_interval = self.broker_properties.get("retry_interval")
                 if not retry_interval:
                     log.warning(
@@ -244,9 +252,9 @@ class SolaceMessaging(Messaging):
                     )
                     .build()
                 )
-            elif strategy and strategy == ConnectionStrategy.PARAMETRIZED_RETRY:
+            elif strategy == ConnectionStrategy.PARAMETRIZED_RETRY:
                 retry_count = self.broker_properties.get("retry_count")
-                retry_interval = self.broker_properties.get("retry_wait")
+                retry_interval = self.broker_properties.get("retry_interval")
                 if not retry_count:
                     log.warning(
                         f"{self.error_prefix} retry_count not provided, using default value of 20"
@@ -254,7 +262,7 @@ class SolaceMessaging(Messaging):
                     retry_count = 20
                 if not retry_interval:
                     log.warning(
-                        f"{self.error_prefix} retry_wait not provided, using default value of 3000"
+                        f"{self.error_prefix} retry_interval not provided, using default value of 3000"
                     )
                     retry_interval = 3000
                 self.messaging_service = (
@@ -268,23 +276,6 @@ class SolaceMessaging(Messaging):
                     )
                     .build()
                 )
-            else:
-                # set default
-                log.info(
-                    f"{self.error_prefix} Using default reconnection strategy. 20 retries with 3000ms interval"
-                )
-                strategy = ConnectionStrategy.PARAMETRIZED_RETRY
-                self.messaging_service = (
-                    MessagingService.builder()
-                    .from_properties(broker_props)
-                    .with_reconnection_retry_strategy(
-                        RetryStrategy.parametrized_retry(retry_count, retry_interval)
-                    )
-                    .with_connection_retry_strategy(
-                        RetryStrategy.parametrized_retry(retry_count, retry_interval)
-                    )
-                    .build()
-                )
 
             # Blocking connect thread
             result = self.messaging_service.connect_async()
@@ -301,7 +292,7 @@ class SolaceMessaging(Messaging):
                     or result.done()
                 ):
                     # update retry count
-                    if strategy and strategy == ConnectionStrategy.PARAMETRIZED_RETRY:
+                    if strategy == ConnectionStrategy.PARAMETRIZED_RETRY:
                         if temp_retry_count <= 0:
                             log.error(
                                 f"{self.error_prefix} Connection attempts exhausted. Stopping..."
@@ -326,7 +317,7 @@ class SolaceMessaging(Messaging):
             if self.stop_signal.is_set():
                 log.error(f"{self.error_prefix} Stopping connection attempt")
                 self.disconnect()
-                raise KeyboardInterrupt("Stopping connection attempt")
+                raise KeyboardInterrupt("Stopping connection attempt") from None
 
             self.stop_connection_log.set()
             log.info(f"{self.error_prefix} Successfully connected to broker.")
@@ -372,9 +363,12 @@ class SolaceMessaging(Messaging):
                     self.broker_properties.get("subscriptions"),
                     self.broker_properties.get("temporary_queue"),
                     self.broker_properties.get("max_redelivery_count"),
+                    self.broker_properties.get("create_queue_on_start"),
                 )
         except KeyboardInterrupt:  # pylint: disable=broad-except
-            raise KeyboardInterrupt
+            raise KeyboardInterrupt from None
+        except Exception as e:
+            raise ValueError("Error in broker connection") from None
 
     def bind_to_queue(
         self,
@@ -382,6 +376,7 @@ class SolaceMessaging(Messaging):
         subscriptions: list = None,
         temporary: bool = False,
         max_redelivery_count: int = None,
+        create_queue_on_start: bool = True,
     ):
         if subscriptions is None:
             subscriptions = []
@@ -391,12 +386,22 @@ class SolaceMessaging(Messaging):
         else:
             queue = Queue.durable_exclusive_queue(queue_name)
 
+        # Create a queue if create_queue_on_start is set to True
+        if create_queue_on_start:
+            missing_resources_creation_strategy = (
+                MissingResourcesCreationStrategy.CREATE_ON_START
+            )
+        else:
+            missing_resources_creation_strategy = (
+                MissingResourcesCreationStrategy.DO_NOT_CREATE
+            )
+
         try:
             # Build a receiver and bind it to the queue
             self.persistent_receiver: PersistentMessageReceiver = (
                 self.messaging_service.create_persistent_message_receiver_builder()
                 .with_missing_resources_creation_strategy(
-                    MissingResourcesCreationStrategy.CREATE_ON_START
+                    missing_resources_creation_strategy
                 )
                 .with_required_message_outcome_support(
                     Message_NACK_Outcome.FAILED, Message_NACK_Outcome.REJECTED
@@ -416,7 +421,8 @@ class SolaceMessaging(Messaging):
                     }
                 except Exception as e:
                     log.error(
-                        f"{self.error_prefix} Error setting max redelivery count: {str(e)}"
+                        f"{self.error_prefix} Error setting max redelivery count",
+                        trace=e,
                     )
 
             self.persistent_receiver.start()
@@ -428,18 +434,18 @@ class SolaceMessaging(Messaging):
             )
 
         # Handle API exception
-        except PubSubPlusClientError as exception:
+        except PubSubPlusClientError:
             log.warning(
-                f"{self.error_prefix} Error creating persistent receiver for queue [%s], %s",
+                f"{self.error_prefix} Error creating persistent receiver for queue [%s]",
                 queue_name,
-                exception,
             )
+            raise exception
 
         # Add to list of receivers
         self.persistent_receivers.append(self.persistent_receiver)
 
         # If subscriptions are provided, add them to the receiver
-        if subscriptions:
+        if create_queue_on_start and subscriptions:
             for subscription in subscriptions:
                 sub = TopicSubscription.of(subscription.get("topic"))
                 self.persistent_receiver.add_subscription(sub)
@@ -453,8 +459,8 @@ class SolaceMessaging(Messaging):
             change_connection_status(
                 self.connection_properties, ConnectionStatus.DISCONNECTED
             )
-        except Exception as exception:  # pylint: disable=broad-except
-            log.debug(f"{self.error_prefix} Error disconnecting: %s", exception)
+        except Exception as e:  # pylint: disable=broad-except
+            log.error(f"{self.error_prefix} Error disconnecting", trace=e)
 
     def get_connection_status(self):
         return self.connection_properties["status"]
@@ -505,6 +511,76 @@ class SolaceMessaging(Messaging):
         sub = TopicSubscription.of(subscription)
         persistent_receiver.add_subscription(sub)
 
+    def add_topic_subscription(
+        self, topic_str: str, persistent_receiver: PersistentMessageReceiver
+    ):
+        """Adds a single topic subscription to the specified active PersistentMessageReceiver."""
+        if not self.messaging_service or not self.messaging_service.is_connected:
+            log.error(
+                f"{self.error_prefix} Cannot add subscription. Messaging service not connected."
+            )
+            return False
+        if not persistent_receiver or not persistent_receiver.is_running:
+            log.error(
+                f"{self.error_prefix} Cannot add subscription. Persistent receiver is not valid or not running."
+            )
+            return False
+        try:
+            sdk_topic_subscription = TopicSubscription.of(topic_str)
+            persistent_receiver.add_subscription(sdk_topic_subscription)
+            log.info(
+                f"{self.error_prefix} Successfully added subscription '{topic_str}' to receiver."
+            )
+            return True
+        except PubSubPlusClientError as e:
+            log.error(
+                f"{self.error_prefix} Error adding subscription '{topic_str}'.",
+                trace=e,
+            )
+            return False
+        except Exception as e:
+            log.error(
+                f"{self.error_prefix} Unexpected error adding subscription '{topic_str}'.",
+                exc_info=True,
+                trace=e,
+            )
+            return False
+
+    def remove_topic_subscription(
+        self, topic_str: str, persistent_receiver: PersistentMessageReceiver
+    ):
+        """Removes a single topic subscription from the specified active PersistentMessageReceiver."""
+        if not self.messaging_service or not self.messaging_service.is_connected:
+            log.error(
+                f"{self.error_prefix} Cannot remove subscription. Messaging service not connected."
+            )
+            return False
+        if not persistent_receiver or not persistent_receiver.is_running:
+            log.error(
+                f"{self.error_prefix} Cannot remove subscription. Persistent receiver is not valid or not running."
+            )
+            return False
+        try:
+            sdk_topic_subscription = TopicSubscription.of(topic_str)
+            persistent_receiver.remove_subscription(sdk_topic_subscription)
+            log.info(
+                f"{self.error_prefix} Successfully removed subscription '{topic_str}' from receiver."
+            )
+            return True
+        except PubSubPlusClientError as e:
+            log.error(
+                f"{self.error_prefix} Error removing subscription '{topic_str}'.",
+                trace=e,
+            )
+            return False
+        except Exception as e:
+            log.error(
+                f"{self.error_prefix} Unexpected error removing subscription '{topic_str}'.",
+                exc_info=True,
+                trace=e,
+            )
+            return False
+
     def ack_message(self, broker_message):
         if "_original_message" in broker_message:
             self.persistent_receiver.ack(broker_message["_original_message"])
