@@ -16,9 +16,10 @@ from ...common.utils import ensure_slash_on_end, ensure_slash_on_start
 info = {
     "class_name": "BrokerRequestResponse",
     "description": (
-        "Connect to a messaging broker, send request messages, and receive responses. "
-        "This component combines the functionality of broker_input and broker_output "
-        "with additional request-response handling."
+        "This component sends request messages to a broker and waits for correlated responses, "
+        "handling both outbound requests and inbound responses within a single component. "
+        "This is performed asynchronously, allowing to to handle multiple requests and responses "
+        "at the same time. "
     ),
     "config_parameters": [
         {
@@ -311,7 +312,7 @@ class BrokerRequestResponse(BrokerBase):
                 if broker_message:
                     self.process_response(broker_message)
             except Exception as e:
-                log.error("Error handling response: %s", e)
+                log.error("Error handling response.", trace=e)
 
     def handle_test_pass_through(self):
         while not self.stop_signal.is_set():
@@ -320,10 +321,11 @@ class BrokerRequestResponse(BrokerBase):
                 decoded_payload = self.decode_payload(message.get_payload())
                 message.set_payload(decoded_payload)
                 self.process_response(message)
-            except queue.Empty:
+            except queue.Empty as e:
+                log.debug("No messages in pass-through queue.", trace=e)
                 continue
             except Exception as e:
-                log.error("Error handling test passthrough: %s", e)
+                log.error("Error handling test passthrough.", trace=e)
 
     def process_response(self, broker_message):
         if self.test_mode:
@@ -337,9 +339,9 @@ class BrokerRequestResponse(BrokerBase):
             user_properties = broker_message.get("user_properties", {})
 
             self.messaging_service.ack_message(broker_message)
-        
+
         if not user_properties:
-            log.error("Received response without user properties: %s", payload)
+            log.error("Received response without user properties.")
             return
 
         streaming_complete_expression = None
@@ -347,34 +349,32 @@ class BrokerRequestResponse(BrokerBase):
             user_properties, self.user_properties_reply_metadata_key, True
         )
         if not metadata_json:
-            log.error("Received response without metadata: %s", payload)
+            log.error("Received response without metadata.")
             return
 
         try:
             metadata_stack = json.loads(metadata_json)
-        except json.JSONDecodeError:
-            log.error("Received response with invalid metadata JSON: %s", metadata_json)
+        except json.JSONDecodeError as e:
+            log.error("Received response with invalid metadata JSON.", trace=e)
             return
 
         if not metadata_stack:
-            log.error("Received response with empty metadata stack: %s", payload)
+            log.error("Received response with empty metadata stack.")
             return
 
         try:
             current_metadata = metadata_stack.pop()
-        except IndexError:
-            log.error(
-                "Received response with invalid metadata stack: %s", metadata_stack
-            )
+        except IndexError as e:
+            log.error("Received response with invalid metadata stack.", trace=e)
             return
         request_id = current_metadata.get("request_id")
         if not request_id:
-            log.error("Received response without request_id in metadata: %s", payload)
+            log.error("Received response without request_id in metadata.")
             return
 
         cached_request = self.cache_service.get_data(request_id)
         if not cached_request:
-            log.error("Received response for unknown request_id: %s", request_id)
+            log.error("Received response for unknown request_id.")
             return
 
         stream = cached_request.get("stream", False)
@@ -424,6 +424,12 @@ class BrokerRequestResponse(BrokerBase):
             is_last = message.get_data(streaming_complete_expression)
             if not is_last:
                 last_piece = False
+                self.cache_service.add_data(
+                    key=request_id,
+                    value=cached_request,
+                    expiry=self.request_expiry_ms / 1000,  # Reset expiry time
+                    component=self,
+                )
 
         if last_piece:
             self.cache_service.remove_data(request_id)
@@ -457,14 +463,9 @@ class BrokerRequestResponse(BrokerBase):
                     existing_metadata.append(metadata)
                     metadata = existing_metadata
                 else:
-                    log.warning(
-                        "Invalid existing metadata format: %s", existing_metadata
-                    )
+                    log.warning("Invalid existing metadata format.")
             except json.JSONDecodeError:
-                log.warning(
-                    "Failed to decode existing metadata JSON: %s",
-                    existing_metadata_json,
-                )
+                log.warning("Failed to decode existing metadata JSON.")
         else:
             metadata = [metadata]
 
@@ -495,7 +496,9 @@ class BrokerRequestResponse(BrokerBase):
             if self.broker_type == "test_streaming":
                 # The payload should be an array. Send one message per item in the array
                 if not isinstance(data["payload"], list):
-                    raise ValueError("Payload must be a list for test_streaming broker")
+                    raise ValueError(
+                        "Payload must be a list for test_streaming broker"
+                    ) from None
                 for item in data["payload"]:
                     encoded_payload = self.encode_payload(item)
                     self.pass_through_queue.put(
